#!/bin/bash

# Hotfix script for TYPO3 bug #76231
#
# This script will search for affected TYPO3 CMS versions within ROOTPATH (can be changed
# on runtime) and tries to apply patch files automatically.
#
# Author: Nicole Cordes <typo3@cordes.co>
# For questions, please contact <security@typo3.org>
#
# Usage:
# 1) Make this script executable
# 2) Run this script
#
# chmod a+x ./fix_76231.sh
# ./fix_76231.sh [rootPath]
#

export LC_ALL=C

if [[ -z "${1}"  || ! -d "${1}" ]]; then
	read -e -p "Please enter the path to TYPO3 root [/var/www/]: " ROOTPATH
	[ -z "${ROOTPATH}" ] && ROOTPATH="/var/www/"
else
	ROOTPATH="${1}"
fi

_check_required_commands() {
    local missed=""

    until [ -z "$1" ]; do
        type -t $1 >/dev/null 2>/dev/null
        if [ $? -ne 0 ]; then
            missed="$missed $1"
        fi
        shift
    done

    echo $missed
}

REQUIRED_COMMANDS='patch'
MISSED_REQUIRED_COMMANDS=$(_check_required_commands $REQUIRED_COMMANDS)
if [ ! -z "${MISSED_REQUIRED_COMMANDS}" ]
then
    echo "Error! Some required system tools, that are utilized in this script, are not installed:" >&2
    echo "Tools \"${MISSED_REQUIRED_COMMANDS}\" are missed, please install them." >&2
    exit 1
fi

find -L "${ROOTPATH}" -type f -name "SystemEnvironmentBuilder.php" -print -o -type f -name "config_default.php" -print 2>/dev/null | while read FILE; do
	line=$(grep -oE "define\('TYPO3_branch', '[0-9.]+" "${FILE}" 2>/dev/null)
	if [ -z "${line}" ]; then
		continue
	fi

	# Get necessary variables from grep statement
	TYPO3BRANCH=$(echo "${line}" | cut -d "'" -f 4)
	TYPO3MAINVERSION=$(echo "${TYPO3BRANCH}" | cut -d '.' -f 1)

	# Extract TYPO3 root path from file depending on TYPO3 main version
	if [ "${TYPO3MAINVERSION}" -lt "6" ]; then
		TYPO3ROOTPATH=${FILE%/t3lib/*}
	else
		TYPO3ROOTPATH=${FILE%/typo3/sysext/core/*}
	fi

	# Continue if typo3 directory is a symlink
	if [ -h "${TYPO3ROOTPATH}/typo3" ]; then
		continue
	fi

	# Get patch file suffix
	case ${TYPO3BRANCH} in
		4.3|4.4|4.5|4.6|4.7|6.0|6.1|6.2|7.0|7.6)
			PATCHFILESUFFIX=${TYPO3BRANCH/./-}
			;;
		7.1|7.2|7.3|7.4|7.5)
			PATCHFILESUFFIX=7
			;;
		8.0|8.1|8.2)
			PATCHFILESUFFIX=8
			;;
		*)
			PATCHFILESUFFIX=
	esac

	if [ -z "${PATCHFILESUFFIX}" ]; then
		echo "No patch found for TYPO3 CMS version ${TYPO3BRANCH} in ${TYPO3ROOTPATH}" >&2
		continue
	fi

	PATCHFILE=$(cd "$(dirname "$0")" && pwd)/TYPO3_${PATCHFILESUFFIX}.diff

	patch -p1 -f --dry-run -d "${TYPO3ROOTPATH}" < ${PATCHFILE} >/dev/null 2>&1
	if [ $? -eq 0 ]; then
		patch -p1 -f -d "${TYPO3ROOTPATH}" < ${PATCHFILE} >/dev/null 2>&1
		echo "Patch for TYPO3 CMS version ${TYPO3BRANCH} applied in ${TYPO3ROOTPATH}"
	else
		patch -p1 -f -R --dry-run -d "${TYPO3ROOTPATH}" < ${PATCHFILE} >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			echo "Patch for TYPO3 CMS version ${TYPO3BRANCH} already applied in ${TYPO3ROOTPATH}"
		else
			echo "Patch for TYPO3 CMS version ${TYPO3BRANCH} could not be applied in ${TYPO3ROOTPATH}" >&2
		fi
	fi
done
